/*
 * $Revision: 1.0 $
 */

#include <stdio.h>
#include <string.h>
#include <inarm.h>
#include <iolpc2114.h>
#include "lpc221x.h"
#include "timer.h"
#include "flash_lpc.h"
#include "config.h"

#define FLASH_ERROR 0x01
#define RAM_ERROR   0x02
#define UART_ERROR  0x04

#define CRYSTAL_SEMI_ID     0x630E
#define CRYSTAL_LAN_REV_B   0x0700
#define CRYSTAL_LAN_REV_C   0x0800
#define CRYSTAL_LAN_REV_D   0x0900
#define CRYSTAL_LAN_REV_F   0x0A00

#define	LCD_CHAR_SPEED  2000          //2ms
#define CAN_DELAY_TIME  10
#define OFF             0
#define ON              1  

#define TEST_NUMB   68

// LCD pinout
#define   E_high          IO0SET |= 0x20000000
#define   E_low           IO0CLR |= 0x20000000
#define   RS_high         IO0SET |= 0x10000000
#define   RS_low          IO0CLR |= 0x10000000
#define   RW_high         IO0SET |= 0x40000000
#define   RW_low          IO0CLR |= 0x40000000
#define   LIGHT_LCD_ON    IO0SET |= 0x00000400
#define   LIGHT_LCD_OFF   IO0CLR |= 0x00000400
#define   TD2_high        IO0SET |= 0x01000000
#define   TD2_low         IO0CLR |= 0x01000000
#define   TD3_high        IO0SET |= 0x00400000
#define   TD3_low         IO0CLR |= 0x00400000
#define   TD4_high        IO0SET |= 0x00002000
#define   TD4_low         IO0CLR |= 0x00002000

#define   B1              IO0PIN & 0x00008000
#define   B2              IO0PIN & 0x00010000
#define   RD1             IO0PIN & 0x02000000
#define   RD2             IO0PIN & 0x00800000
#define   RD3             IO0PIN & 0x00200000
#define   RD4             IO0PIN & 0x00001000
#define   AIN0            IO0PIN & 0x08000000
#define   DLS             IO0PIN & 0x00000800


#define		DISP_ON		0x0000000C	//LCD control constants
#define		DISP_OFF	0x00000008	//
#define		CLR_DISP	0x00000001	//
#define		CUR_HOME	0x00000002	//
#define		ENTRY_INC	0x00000006	//
#define		DD_RAM_ADDR	0x00000080	//
#define		DD_RAM_ADDR2	0x000000C0	//  
#define		DD_RAM_ADDR3	0x00000028	//      
#define		CG_RAM_ADDR	0x00000040	// 

/*
#define         I2EN            I2CONSET |= 0x40
#define         STA             I2CONSET |= 0x20 
#define         STO             I2CONSET |= 0x10
#define         SI              I2CONSET |= 0x80
#define         AA              I2CONSET |= 0x04

#define         I2ENC           I2CONCLR |= 0x40
#define         STAC            I2CONCLR |= 0x20 
#define         STOC            I2CONCLR |= 0x10
#define         SIC             I2CONCLR |= 0x80
#define         AAC             I2CONCLR |= 0x04
*/

#define         SCL             IO0PIN & 0x00000004
#define         SDA             IO0PIN & 0x00000008


const unsigned char Message_board_name[] = " LPC-E22xx board  by OLIMEX Ltd ";
const unsigned char MessageALABALA[] = " Visit the site  www.olimex.com ";
const unsigned char MessageBEGIN_TEST[] = "   BOARD TEST   ";
const unsigned char MessageTEST_SUCCESS[] = "  TEST SUCCESS  ";

unsigned char i,j,b1,b2;
unsigned long temp,data;
unsigned char flag_SPI_ERR = 0;
unsigned char flag_I2C_ERR = 0;
unsigned char CAN2_ERR = 0;
unsigned char CAN3_ERR = 0;
unsigned char CAN4_ERR = 0;

void Delay_us (unsigned long a);
void _E (void);
void LCD_Ini(void);
void SEND_CHAR (unsigned long c);
void SEND_CMD (unsigned long c);
unsigned char sendByte1(char byte);
unsigned char getByte1(char *byte);
void LPC210xUART1TxByte(unsigned char byte);
void LPC210xInitUART1();
void WriteWord(unsigned long Add, unsigned short usData);
unsigned short ReadWord(unsigned long Add);
void InitCS8900A ();
void sendMessage_UART1(char * byte);
void sendMessage_UART0(char * byte);

const unsigned long RamTest [TEST_NUMB] = {

  0xFFFFFFFF,
  0xFFFFFFFE,
  0xFFFFFFFC,
  0xFFFFFFF8,
  0xFFFFFFF0,

  0xFFFFFFEF,
  0xFFFFFFCF,
  0xFFFFFF8F,
  0xFFFFFF0F,

  0xFFFFFEFF,
  0xFFFFFCFF,
  0xFFFFF8FF,
  0xFFFFF0FF,

  0xFFFFEFFF,
  0xFFFFCFFF,
  0xFFFF8FFF,
  0xFFFF0FFF,

  0xFFFEFFFF,
  0xFFFCFFFF,
  0xFFF8FFFF,
  0xFFF0FFFF,

  0xFFEFFFFF,
  0xFFCFFFFF,
  0xFF8FFFFF,
  0xFF0FFFFF,

  0xFEFFFFFF,
  0xFCFFFFFF,
  0xF8FFFFFF,
  0xF0FFFFFF,

  0xEFFFFFFF,
  0xCFFFFFFF,
  0x8FFFFFFF,
  0x0FFFFFFF,

  ~0xFFFFFFFF,
  ~0xFFFFFFFE,
  ~0xFFFFFFFC,
  ~0xFFFFFFF8,
  ~0xFFFFFFF0,

  ~0xFFFFFFEF,
  ~0xFFFFFFCF,
  ~0xFFFFFF8F,
  ~0xFFFFFF0F,

  ~0xFFFFFEFF,
  ~0xFFFFFCFF,
  ~0xFFFFF8FF,
  ~0xFFFFF0FF,

  ~0xFFFFEFFF,
  ~0xFFFFCFFF,
  ~0xFFFF8FFF,
  ~0xFFFF0FFF,

  ~0xFFFEFFFF,
  ~0xFFFCFFFF,
  ~0xFFF8FFFF,
  ~0xFFF0FFFF,

  ~0xFFEFFFFF,
  ~0xFFCFFFFF,
  ~0xFF8FFFFF,
  ~0xFF0FFFFF,

  ~0xFEFFFFFF,
  ~0xFCFFFFFF,
  ~0xF8FFFFFF,
  ~0xF0FFFFFF,

  ~0xEFFFFFFF,
  ~0xCFFFFFFF,
  ~0x8FFFFFFF,
  ~0x0FFFFFFF,

  0xAAAAAAAA,
  0x55555555,
};

unsigned char sendByte0(char byte)
{
  while(!U0LSR_bit.THRE);
  LPC210xUART0TxByte(byte);
  return 1;
}

unsigned char sendByte1(char byte)
{
  while(!U1LSR_bit.THRE);
  LPC210xUART1TxByte(byte);
  return 1;
}

void sendMessage_UART0(char * byte)
{
  for(;*byte;++byte)
  {
    sendByte0(*byte);
  }
}

void sendMessage_UART1(char * byte)
{
  for(;*byte;++byte)
  {
    sendByte1(*byte);
  }
}

unsigned char getByte0(char *byte)
{
  if(U0LSR_bit.DR)
  {
    *byte = U0RBR;
    return 1;
  }
  return 0;
}

unsigned char getByte1(char *byte)
{
  if(U1LSR_bit.DR)
  {
    *byte = U1RBR;
    return 1;
  }
  return 0;
}


void Delay_us(unsigned long a)      //Delay in us
{
    unsigned long n;
    n=a*10;
    while (n-- != 0);
}

void _E (void)
{
	E_high;
	Delay_us(10);
	E_low;
}	

//-----LCD_ini------------------
void LCD_Ini(void)
{
        LIGHT_LCD_OFF;
	RS_low;		
        RW_low;
        E_low;		
	Delay_us(110000);	//110ms
	IO0SET |= 0x30;		
	_E();	
	Delay_us(10000);	
	IO0SET |= 0x30;
	_E();
	Delay_us(10000);
	IO0SET |= 0x30;
	_E();
	Delay_us(10000);
	IO0SET |= 0x20;
        IO0CLR |= 0x10;
	_E();
} 
//--------Send char to LCD-----------------------------
void SEND_CHAR (unsigned long c)
{    
        RW_low;
	Delay_us(LCD_CHAR_SPEED);              	    //Delay for LCD char
	data = 0xffffff0f | c;			    //get upper nibble
        IO0CLR |= 0x000000f0;                        //clear d4-d7        
	IO0SET = (IO0SET | 0x000000f0) & data;        //set D4-D7
	RS_high;	  			    //set LCD to data mode
	_E();
	data = (c << 4) & 0x000000f0;   	    //get down nibble
        IO0CLR |= 0x000000f0;                        //clear D4-D7
	IO0SET = (IO0SET & 0xffffff0f) | data;	    //set D4-D7 (only PORTC4-PORTC7)
	RS_high;        			    //set LCD to data mode
	_E();
}

//--------Send command to LCD-----------------------------
void SEND_CMD (unsigned long c)
{    
        RW_low;
	Delay_us(LCD_CHAR_SPEED);              	    //Delay for LCD char
	data = 0xffffff0f | c;			    //get upper nibble
        IO0CLR |= 0x000000f0;                        //clear d4-d7        
	IO0SET = (IO0SET | 0x000000f0) & data;        //set D4-D7
	RS_low;         			    //set LCD to data mode
	_E();
	data = (c << 4) & 0x000000f0;   	    //get down nibble
        IO0CLR |= 0x000000f0;                        //clear D4-D7
	IO0SET = (IO0SET & 0xffffff0f) | data;	    //set D4-D7 (only PORTC4-PORTC7)
	RS_low;         			    //set LCD to data mode
	_E();
}

//CS8900
unsigned long pTemp;
unsigned char *pAdd_CS = (unsigned char *)CS8900A_BASE;

void InitCS8900A ()
{
  // FLASH 16bit; IDCY = F; WST1 = 4; RBLE = 1; WST2 = 6
 // BCFG2=0x00003482;
  // CS8900A 8bit; 
  BCFG2_bit.MW = 0;
  // IDCY = 1+1 ~ 33ns 
  BCFG2_bit.IDCY=1;
  // WST1 = 8+3 (read)  ~183ns
  BCFG2_bit.WST1=8;
  // WST2 = 6+1 (write) ~ 117ns
  BCFG2_bit.WST2=6;
  // RBLE = 1 - ako e 0 ne zapiswa
  BCFG2_bit.RBLE = 1;
  // WP =0, BM = 0
  BCFG2_bit.WP = BCFG2_bit.BM = 0;

  // P3.25 - CS2 (CS8900A)
  // P3.0 - A0
  PINSEL2 |= 0x00804000;
}

void WriteWord(unsigned long Add, unsigned short uiData)
{
  pAdd_CS = (unsigned char*)(CS8900A_BASE+Add);
  *pAdd_CS = uiData;
  ++pAdd_CS;
  *pAdd_CS = uiData>>8;
}

unsigned short ReadWord(unsigned long Add)
{
unsigned short Data;
  pAdd_CS = (unsigned char*)(CS8900A_BASE+Add);
  Data = *pAdd_CS;
  Data |= ((*++pAdd_CS) << 8);
  return(Data);
}

unsigned int led = 0, id;
unsigned long* pAdd;

unsigned short Data;

void main(void)
{
unsigned long i,j = 0;
unsigned long hold;
unsigned int error_status;
char byte;
char Message[256];
unsigned short Data;
int CrystalLan = 0;

b1=b2=OFF;

FLASH_DEV_MANUFACTURER_ID_DEF FlashDevManID = UNKNOW_MAN_ID;
    
  T0TC = 0;
  // System initialization, this will map the exception vectors.
  LPC210xSystemInit();
  
  // Set up peripheral registers.
  LPC210xInitPIO();
  
  // First disable interrupts.
  __disable_interrupt();
  // Setup interrupt controller.
  LPC210xInitVIC();

  // Periodic timer initialization.
  LPC210xInitTimer();

//  LPC210xInitTimerInterrupt(TimerBeat);

  //Set up UART0
  LPC210xInitUART0();
  
  //Set up UART1
  LPC210xInitUART1();
    
  // Enable interrupts.
  __enable_interrupt();

  // Start periodic timer.
  LPC210xStartTimer();
  
 // Init CS8900A
  InitCS8900A();  
/* 
 //I2C ini
 I2SCLH=500;
 I2SCLL=500;
 I2EN;
 
 while(1)  
  {    
    I2DAT=0x55;    
  }     
*/  

  IO0CLR |= 0x001e0000;      //0b 0000 0000 0001 1110 0000 0000 0000 0000   SPI  
  IO0DIR |= 0x00020000;      //0b 0000 0000 0000 0010 0000 0000 0000 0000   SCK1  is output
  if ((IO0PIN & 0x001e0000) != 0x001C0000) flag_SPI_ERR=1;
  IO0DIR &= ~0x00020000;    //SCK1 is input
  IO0DIR |=0x00040000;      //0b 0000 0000 0000 0100 0000 0000 0000 0000  MISO is output
  if ((IO0PIN & 0x001e0000) != 0x001A0000) flag_SPI_ERR=1;
  IO0DIR &= ~0x00040000;    //MISO is input
  IO0DIR |= 0x00080000;      //0b 0000 0000 0000 1000 0000 0000 0000 0000    MOSI is output
  if ((IO0PIN & 0x001e0000) != 0x00160000) flag_SPI_ERR=1;
  IO0DIR &= ~0x00080000;  //MOSI is input
  IO0DIR |= 0x00100000;      //0b 0000 0000 0001 0000 0000 0000 0000 0000  SSEL is output  
  if ((IO0PIN & 0x001e0000) != 0x000E0000) flag_SPI_ERR=1;
  if (flag_SPI_ERR != 0) sendMessage_UART1("SPI_ERR\n\r");  
  IO0DIR &= ~0x00100000;  //SSEL is INPUT
  
  if ((SDA)==0 || (SCL)==0) flag_I2C_ERR=1;  
  IO0CLR |= 0x0000000C;
  IO0DIR |= 0x00000004;    //SCL is output
  Delay_us(10);
  if ((SDA) == 0 || (SCL) !=0 ) flag_I2C_ERR=1;    
  IO0DIR &= ~0x00000004;    //SCL is input
  IO0DIR |= 0x00000008;    //SDA is output    
  Delay_us(10);
  if ((SDA) != 0 || (SCL) == 0) flag_I2C_ERR=1;  
  IO0DIR &= ~0x00000008;    //SDA is input
  if (flag_I2C_ERR != 0) sendMessage_UART1("I2C_ERR\n\r");   
      
//LCD ini
  LCD_Ini();
  SEND_CMD(DISP_ON);
  SEND_CMD(CLR_DISP);
  SEND_CMD(DD_RAM_ADDR);
  for (i=0 ; i!=16; i++)      			 //Send Message "   BOARD TEST   " to LCD
      {
	temp=MessageBEGIN_TEST[i];
	SEND_CHAR(temp);  
      }
    
  // CS8900A test
  sendMessage_UART1("CS8900A Test\n\r");
  WriteWord(0xA,0x0);
  Data = ReadWord(0xC);
  if (Data == CRYSTAL_SEMI_ID) {
    sendMessage_UART1("Crystal LAN is find\n\r");
    CrystalLan = 1;
  } else {
    sprintf(Message,"%s %x %s","Unknow Manufacturer ID - ",Data,"\n\r");
    sendMessage_UART1(Message);
  }
  WriteWord(0xA,0x2);
  Data = ReadWord(0xC);
  if ((Data == CRYSTAL_LAN_REV_B) && CrystalLan) {
    sendMessage_UART1("Crystal LAN rev - B\n\r");
  } else if ((Data == CRYSTAL_LAN_REV_C) && CrystalLan) {
    sendMessage_UART1("Crystal LAN rev - C\n\r");
  } else if ((Data == CRYSTAL_LAN_REV_D) && CrystalLan) {
    sendMessage_UART1("Crystal LAN rev - D\n\r");
  } else if ((Data == CRYSTAL_LAN_REV_F) && CrystalLan) {
    sendMessage_UART1("Crystal LAN rev - F\n\r");
  } else {
    sprintf(Message,"%s %x %s","Unknow Revision - ",Data,"\n\r");
    sendMessage_UART1(Message);
  }    

//CAN_TEST

//  sendMessage_UART0("CAN Tests\n\r");
  CAN2_ERR=CAN3_ERR=CAN4_ERR=0;
  if ((RD1) == 0) sendMessage_UART0("CAN1 error\n\r");    //vnimanie CAN1 e razmenen s CAN2
  TD2_low;
  Delay_us(CAN_DELAY_TIME);
  if ((RD2) != 0) CAN2_ERR=1;
  TD2_high;
  Delay_us(CAN_DELAY_TIME);
  if ((RD2) == 0) CAN2_ERR=1;
  if (CAN2_ERR !=0 ) sendMessage_UART0("CAN2 error\n\r");
  TD3_low;
  Delay_us(CAN_DELAY_TIME);
  if ((RD3) != 0) CAN3_ERR = 1;
  TD3_high;
  Delay_us(CAN_DELAY_TIME);
  if ((RD3) == 0) CAN3_ERR = 1;
  if (CAN3_ERR != 0) sendMessage_UART0("CAN3 error\n\r");      
  TD4_low;
  Delay_us(CAN_DELAY_TIME);
  if ((RD4) != 0) CAN4_ERR = 1;
  TD4_high;
  Delay_us(CAN_DELAY_TIME);
  if ((RD4) == 0) CAN4_ERR = 1;
  if (CAN4_ERR != 0) sendMessage_UART0("CAN4 error\n\r");    
  
  if (CAN2_ERR == 0 && CAN3_ERR == 0 && CAN4_ERR == 0) sendMessage_UART0("CAN Tests SUCCESS\n\r");  
 
  
  // Init error status 
  error_status = 0;  

  // Ram memory test
//  sendMessage_UART0("SRAM Tests\n\r");
//  sendMessage_UART0("SRAM Write\n\r");

#if TRACE_ENA
  printf(" SRAM Tests\n");
  printf("SRAM Write\n");
#endif
  for(i=0, pAdd=(unsigned long *)SRAM_BASE; pAdd < (unsigned long *)SRAM_BASE+SRAM_SIZE/4; ++pAdd,++i)
  {
    *pAdd = i;
  }

  sendMessage_UART0("SRAM Write Pass\n\r");
  sendMessage_UART0("SRAM Address Bus Verify\n\r");

#if TRACE_ENA
  printf("%s","SRAM Write Pass\n");
  printf("%s","SRAM Address Bus Verify\n");
#endif
  for(i=0, pAdd=(unsigned long *)SRAM_BASE; pAdd < (unsigned long *)SRAM_BASE+SRAM_SIZE/4; ++pAdd,++i)
  {
    if (*pAdd != i)
      break;
  }
  if (pAdd >= (unsigned long *)SRAM_BASE+SRAM_SIZE/4)
  {
      sendMessage_UART0("SRAM Adderss Bus Verify Pass\n\r");
  
    #if TRACE_ENA
      printf("%s","SRAM Adderss Bus Verify Pass\n");
    #endif
  }
  else
  {
    sprintf(Message,"%s %#x %s","SRAM Address Bus Write Fault at address - ",pAdd,"\n\r");
    sendMessage_UART0(Message);

  #if TRACE_ENA
    printf("SRAM Address Bus Write Fault at address - ",pAdd,'\n');
  #endif
    error_status |= RAM_ERROR;
  }

  if((error_status & RAM_ERROR) == 0)
  {
    sendMessage_UART0("SRAM Data Bus Verify\n\r");
    #if TRACE_ENA
      printf("%s","SRAM Data Bus Verify\n");
    #endif
    for(i=TEST_NUMB, pAdd=(unsigned long *)SRAM_BASE; i; --i)
    {
      *pAdd = RamTest[i];
      if (*pAdd != RamTest[i])
        break;
    }
    if (i == 0)
    {
      sendMessage_UART0("SRAM Data Bus Verify Pass\n\r");
  
    #if TRACE_ENA
      printf("%s","SRAM Data Bus Verify Pass\n");
    #endif
    }
    else
    {
      sendMessage_UART0("SRAM Data Bus Verify Fault\n\r");  
    }
  }
  
  // Flash memory test
  sendMessage_UART0("Flash Tests\n\r");
  sendMessage_UART0("Read manufacturer\n\r");

#if TRACE_ENA
  printf("Flash Tests\n");
  printf("Read manufacturer\n");
#endif
  id = FlashID((unsigned short*)(FLASH_BASE)+MANUFACTURER_ID);
  if (id == 0x0089)
  {
    sendMessage_UART0("Intel flash device is find\n\r");
  #if TRACE_ENA
    printf("Intel flash device is find\n");
  #endif
    // Set flash man. type
    FlashDevManID = INTEL_MAN_ID;
  }
  else if (id == 0x00C2)
  {
    sendMessage_UART0("MXIC flash device is find\n\r");
  #if TRACE_ENA
    printf("MXIC flash device is find\n");
  #endif
    // Set flash man. type
    FlashDevManID = MXID_MAN;
  }
  else
  {
    sprintf(Message,"%s %#x %s","Unknow flash device is find",id,"\n\r");
    sendMessage_UART0(Message);

  #if TRACE_ENA
    printf("%s %#x %c","Unknow flash device is find",id,'\n');
  #endif
  }
  // Read type of flash memory
  sendMessage_UART0("Read type of chip\n\r");

#if TRACE_ENA
  printf("Read type of chip\n");
#endif
  id = FlashID((unsigned short*)(FLASH_BASE)+DEVICE_ID);
  if (id == 0x88C5)
  {
    sendMessage_UART0("TE28F320C3Bxxx\n\r");

  #if TRACE_ENA
    printf("TE28F320C3Bxxx\n");
  #endif
  }
  else if (id == 0x225B)
  {
    sendMessage_UART0("MX26LV800Bxxx\n\r");

  #if TRACE_ENA
    printf("TE28F320C3Bxxx\n");
  #endif
  }
  else
  {
    sprintf(Message,"%s %#x %s","Unknow type flash device is find",id,"\n\r");
    sendMessage_UART0(Message);

  #if TRACE_ENA
    printf("%s %#x %c","Unknow type flash device is find",id,'\n');
  #endif

    FlashDevManID = UNKNOW_MAN_ID;
  }

  if (FlashDevManID != UNKNOW_MAN_ID)
  {
    sendMessage_UART0("Flash Erase\n\r");
  
  #if TRACE_ENA
    printf("Flash Erase\n");
  #endif
    for(i=8, pAdd=(unsigned long *)FLASH_BASE; pAdd < (unsigned long *)FLASH_BASE+FLASH_SIZE/4; pAdd += FLASH_BLOCK/4,++i)
    {
      LPC210xLedSet(++j&1);
      if (Flash_Erase_Block(pAdd,FlashDevManID) != PASS)
        break;
    }
    if (pAdd >= (unsigned long *)FLASH_BASE+FLASH_SIZE/4)
    {
      sendMessage_UART0("Flash Erase Pass\n\r");
    
    #if TRACE_ENA
      printf("%s","Flash Erase Pass\n");
    #endif
    }
    else 
    {
      sprintf(Message,"%s %d %s","Erase Block - ",i,"\n\r");
      sendMessage_UART0(Message);
      sendMessage_UART0("Flash Erase fault\n\r");
  
    #if TRACE_ENA
      printf("%s %d %c","Erase Block - ",i,'\n');
      printf("%s","Flash Erase fault\n");
    #endif
      error_status |= FLASH_ERROR;
    }
    if ((error_status &  FLASH_ERROR) == 0)
    {
      sendMessage_UART0("Flash Write\n\r");
  
    #if TRACE_ENA
      printf("%s","Flash Write\n");
    #endif
      for(i=0, pAdd=(unsigned long *)FLASH_BASE; i < FLASH_SIZE/4; ++pAdd,++i)
      {
        if ((i & 0x1FFF) == 0)
          LPC210xLedSet(++j&1);
        if (Flash_Write_Word(pAdd,i,FlashDevManID) != PASS)
          break;
      }
    
      if (pAdd >= (unsigned long *)FLASH_BASE+FLASH_SIZE/4)
      {
        sendMessage_UART0("Flash Write Pass\n\r");
  
      #if TRACE_ENA
        printf("%s","Flash Write Pass\n");
      #endif
      }
      else 
      {
        sprintf(Message,"%s %#x %s","Flash Write Fault at address - ",pAdd,"\n\r");
        sendMessage_UART0(Message);
    
      #if TRACE_ENA
        printf("%s %#x %c","Flash Write Fault at address - ",pAdd,'\n');
      #endif
        error_status |= FLASH_ERROR;
      }
      if ((error_status &  FLASH_ERROR) == 0)
      {
        sendMessage_UART0("Flash Verify Data\n\r");
  
      #if TRACE_ENA
        printf("%s","Flash Verify Data\n");
      #endif
        for(i=0, pAdd=(unsigned long *)FLASH_BASE; i < FLASH_SIZE/4; ++pAdd,++i)
        {
          hold = *pAdd;
          if (hold != i)
            break;
          if ((i & 0x1FFF) == 0)
            LPC210xLedSet(++j&1);
        }
      
        if (pAdd >= (unsigned long *)FLASH_BASE+FLASH_SIZE/4)
        {
          sendMessage_UART0("Flash Verify Pass\n\r");
    
        #if TRACE_ENA
          printf("%s","Flash Verify Pass\n");
        #endif
        }
        else 
        {
          sprintf(Message,"%s %#x %s","Flash Verify Fault at address - ",pAdd,"\n\r");
          sendMessage_UART0(Message);
      
        #if TRACE_ENA
          printf("%s %#x %c","Flash Verify Fault at address - ",pAdd,'\n');
        #endif
          error_status |= FLASH_ERROR;
        }
      }
    }
  } 
  else
  {
    sendMessage_UART0("Unknow type flash device is find. Test fault");
    error_status |= FLASH_ERROR;
  }
  
  sendMessage_UART1("Press any key\n\r");
  sendMessage_UART0("Press any key\n\r");
  
#if TRACE_ENA
  printf("UART Tests\n");
  printf("Press key '1' \n");
#endif
  error_status |= UART_ERROR;
  LPC210xLedSet(0);
  
  SEND_CMD(CLR_DISP);
  SEND_CMD(DD_RAM_ADDR);
  for (i=0 ; i!=16; i++)      			 //Send Message "TEST SUCCESS" to LCD
    {
      temp=MessageTEST_SUCCESS[i];
      SEND_CHAR(temp);  
//      if (i == 15) SEND_CMD(DD_RAM_ADDR2);
    }
  
  // Loop forever.    
  for (;;)
  {
  
    if ((AIN0) == 0) LIGHT_LCD_ON;
    else LIGHT_LCD_OFF;
    
    if ((DLS) == 0)  SEND_CMD(CLR_DISP);    
    
    if ((B1) == 0 && b1==OFF) 
      {
        Delay_us(1000);
        b1=ON;
        SEND_CMD(CLR_DISP);
        SEND_CMD(DD_RAM_ADDR);
        for (i=0 ; i!=32; i++)      			 //Send Message "LPC-E22xx by OLIMEX Ltd." to LCD
          {
	    temp=Message_board_name[i];
	    SEND_CHAR(temp);  
	    if (i == 15) SEND_CMD(DD_RAM_ADDR2);
          }   
      }
    if ((B1) != 0 && b1==ON)
      {
        Delay_us(1000);
        b1=OFF;
      }
    
    if ((B2) == 0 && b2==OFF) 
      {
        Delay_us(1000);
        b2=ON;
        SEND_CMD(CLR_DISP);
        SEND_CMD(DD_RAM_ADDR);
        for (i=0 ; i!=32; i++)      			 //Send Message "Visit the site www.olimex.com" to LCD
          {
	    temp=MessageALABALA[i];
	    SEND_CHAR(temp);  
	    if (i == 15) SEND_CMD(DD_RAM_ADDR2);
          }
      }
    if ((B2) != 0 && b2==ON)
      {
        Delay_us(1000);
        b2=OFF;
      }
//---UART1 echo--------    
    if(getByte1(&byte))
    {
      while(!sendByte1(byte)); // Echo byte
      while(!sendByte1('*'));  // Send *
    }
//---UART0 echo--------        
    if(getByte0(&byte))
    {
      while(!sendByte0(byte)); //Echo byte
      while(!sendByte0('*'));  //Send *
    }
//    Sleep(65); // Display for 65 ms.   
  }  
}

